import { ConsoleLogger, Injectable } from '@nestjs/common';
import { OnGuild, PutArg, PutSession, UseCommand } from 'koishi-nestjs';
import { Session } from 'koishi';
import { RecordService } from '../record/record.service';
import { OneBotBot } from '@koishijs/plugin-adapter-onebot';

@Injectable()
export class CommandService extends ConsoleLogger {
  constructor(private record: RecordService) {
    super('CommandService');
  }

  @OnGuild(...(process.env.TARGET_GUILDS?.split(',') || []))
  @UseCommand('rank [rank:string]', 'Ask for a rank')
  async rankCommand(@PutSession() session: Session, @PutArg(0) rank: string) {
    rank ??= '';
    if (rank.length > 6) {
      this.warn(
        `Skipped giving rank ${rank} to ${session.userId} because too long: ${rank.length}`,
      );
      return;
    }
    if (!(await this.record.isCanGiveRecord(session))) {
      this.warn(
        `Skipped giving rank ${rank} to ${session.userId} because already given within 2 hours.`,
      );
      return;
    }
    this.log(`Giving ${rank} to ${session.userId} in ${session.guildId}`);
    try {
      await (session.bot as OneBotBot).internal.setGroupSpecialTitle(
        session.guildId,
        session.userId,
        rank,
      );
      await this.record.recordGiven(session, rank);
      this.log(`Given ${rank} to ${session.userId} in ${session.guildId}`);
    } catch (e) {
      this.error(
        `Error when giving rank ${rank} to ${session.userId} in ${session.guildId}: ${e.message}`,
      );
    }
  }
}
